/********************************************************************
*                 Copyright (C) 2009 by MoSCoS                      *
*              http://www.physics.uq.edu.au/MoSCoS                  *
*               School of Physics and Mathematics                   *
*            The University of Queensland, Australia.               *
*  Authors: Fernando Alonso-Marroquin, Sergio-Andres Galindo-Torres *
********************************************************************/

#include<iostream>
#include<fstream>
#include<string>
#include<math.h>
#include<stdlib.h>
#include<map>
#include "../../src/Graphs/Graph.h"
#define N 30

using namespace std;


extern Quaternion normalize_rotation(double,Vec3 &);

int main() {
    srand(100000);
    ofstream Lf("angulardt4.txt"),Pf("momentumdt4.txt"),Ef("energydt4.txt");
    int i,j,paso=10000;
    double t,dt=1.e-5,E,Erot,Ekin,Epot;
    Vec3 r,v,ome,I(1,0,0),J(0,1,0),K(0,0,1),Z(0,0,0),L,P;
    Quaternion q;
    CParticle aron[N];
    map<pair<int,int>,CInteracton> Inaron;
    map<pair<int,int>,CInteracton>::iterator it;
    for (i=0;i<N-1;i++) {
          for(j=i+1;j<N;j++) {
            pair<int,int> indexes     = make_pair(i,j);
            CInteracton   interaction = CInteracton(aron[i],aron[j]);
            Inaron.insert(make_pair(indexes,interaction));
        }
    }
    for (i=0;i<N;i++) {
        ome=Vec3((1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX);
        q=normalize_rotation(2*M_PI*(1.*rand())/RAND_MAX,ome);
        ome=0.5*Vec3((1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX);
        //r=-2.*I+1.*(0.5-(1.*rand())/RAND_MAX)*K;
        if(i<N/2) {
            r=-2.*I-1.5*i*I;
            v=0.6*I;
        }
        else {
            r=2.*I+1.5*(i-N/2)*I;
            v=-0.6*I;
        }
        aron[i]=CParticle(1.,1.,0.1,r,v,ome,q,"tetra");
        aron[i].start(dt);
    }


    for(t=0,j=0;t<=40.;t+=dt,j++) {
        for (i=0;i<N;i++) {
            aron[i].startforce();
        }
        for(it=Inaron.begin();it!=Inaron.end();it++) {
            (*it).second.calcForce(dt);
        }
        for (i=0;i<N;i++) {
            aron[i].updatePosition(dt);
            aron[i].updateOrientation(dt);
  	    aron[i].updateVertices();
        }
        if(j%paso==0) {
            P=Vec3(0,0,0);
            L=Vec3(0,0,0);
            E=0;
            Erot=0;
            Ekin=0;
            Epot=0;
            CGraph gp("poli.pov");
            //position of the camera
            //r=17*(cos(0.1*M_PI*(t+4))*I+sin(0.1*M_PI*(t+4))*J)+(10-0.2*t)*K;
            r=(-20+0.6*t)*I + 4*J;
	    Vec3 F=Z; // aim of the camera
	    gp.Set_Camera(r,F);

            for (i=0;i<N;i++) {
                gp.Graph_Particle(aron[i],"Gray");
                P+=aron[i].getp();
                L+=aron[i].getLs();
                Ekin+=aron[i].getEkin();
                Erot+=aron[i].getErot();
                E+=aron[i].getEkin()+aron[i].getErot();
            }
            for(it=Inaron.begin();it!=Inaron.end();it++) {
                //gp.Graph_Full_contact((*it).second,0.05,"Blue");
                Epot+=(*it).second.getEpot();
                E+=(*it).second.getEpot();
            }
            gp.close();
            gp.Animation(j/paso);
            //Pf.precision(15);
            Pf << t<<" "<<P.X()<<" "<<P.Y()<<" "<<P.Z()<<" "<<P.norm()<<endl;
            Lf << t<<" "<<L.X()<<" "<<L.Y()<<" "<<L.Z()<<" "<<L.norm()<<endl;
            Ef << t<<" "<<Ekin<<" "<<Erot<<" "<<Epot<<" "<<E<<endl;

        }
    }
    return 0;
}
