/********************************************************************
*                 Copyright (C) 2009 by MoSCoS                      *
*              http://www.physics.uq.edu.au/MoSCoS                  *
*               School of Physics and Mathematics                   *
*            The University of Queensland, Australia.               *
*  Authors: Fernando Alonso-Marroquin, Sergio-Andres Galindo-Torres *
********************************************************************/

#include<iostream>
#include<fstream>
#include<string>
#include<math.h>
#include<stdlib.h>
#include<map>
#include<string>
//#include "../../src/Graphs/Graph.h"
#include "Graphs/Graph.h"
#define N 4

extern Quaternion normalize_rotation(double,Vec3 &);

using namespace std;

int main ()
{   
    //!randon number
    srand(100000);
    int i,j,paso=500,np=0;
    double t,dt=1.e-4,E,Erot,Ekin,Epot;
    Vec3 r,v,ome,I(1,0,0),J(0,1,0),K(0,0,1),Z(0,0,0),L,P,F;
    ofstream Lf("angular.txt"),Pf("momentum.txt"),Ef("energy.txt"),Ff("flow10x6.txt");
    string s("tetra");
    Quaternion q(1,0,0,0);
    //!creating particle array
    CParticle Particles[N];
    r=-5*K;
    //!creating base of the container
    CParticle wall(20,20,0.6,r,q);
    r=-(5-1.25)*K;
    CParticle cow("Mooo.txt");
    //CParticle cow("plate.txt");
    //CParticle cow("cube.txt");
    ///!string pp("pirinola");
    ///!CParticle pirinola(5.5,1,0.1,Z,Z,Z,q,pp);

    //!creating interactons of particles with walls
    CInteracton Inwall[N];
   
    //!creating interacton with particles 
    map<pair<int,int>,CInteracton> Inaron;
    map<pair<int,int>,CInteracton>::iterator it;
    for (i=0;i<N-1;i++) {
          for(j=i+1;j<N;j++) {
            pair<int,int> indexes     = make_pair(i,j);
            CInteracton   interaction = CInteracton(Particles[i],Particles[j]);
            Inaron.insert(make_pair(indexes,interaction));
        }
    }
    
    //!initialize the interactons and particles 
    for (i=0;i<N;i++) {
        Inwall[i]=CInteracton(wall,Particles[i]);
        ome=Vec3((1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX);
        q=normalize_rotation(2*M_PI*(1.*rand())/RAND_MAX,ome);
        ome=0.5*Vec3((1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX,(1.*rand())/RAND_MAX);
        j=int(sqrt(9));
        r=1.6*(i%j-j/2)*I+1.6*((i/j)%j-j/2)*J+(5.+1.6*(i/9))*K;
        v=Vec3(0,0,0);
        if (i<N-1)
           Particles[i]=CParticle(1.,1.,0.1,r,v,v,q,s);
        else
           Particles[i]=cow;;
        Particles[i].start(dt);
    }
    //!create cow as last particle
 
    v=-5.*K;
    wall.moveBy(v);
 
    //!Granular dynamics loops
    
    for(t=0,j=0;t<=40;t+=dt,j++) {
        np=0;
        //! body forces 
        for (i=0;i<N;i++) {
	    F=-Particles[i].getm()*9.8*K;
            Particles[i].startforce(F);
        }
        //cow.startforce(-100*K);
        //! calculate interparticle forces 
        for(it=Inaron.begin();it!=Inaron.end();it++) {
            pair<int,int> indexes = (*it).first;
            //cout<<"-> interaction "<<indexes.first<<" "<<indexes.second<<endl;
            (*it).second.calcForce(dt);
        }
        //! for all particles 
        for (i=0;i<N;i++) {
            //!calculate wall forces  
            Inwall[i].calcForce(dt);
            
            //!update center of mass according to integration algorithm
            Particles[i].updatePosition(dt);

            //!update the quaternion to integration algorithm
            Particles[i].updateOrientation(dt);

            //!update the vertices of the particle
            Particles[i].updateVertices();

            //! if particles are below a threshold => ignore them
            if (Particles[i].getr().Z()<-5) np++;
        }
        //cow.translation(dt);
        //cow.rotation(dt);
        //!plotting instructions
        if(j%paso==0) {
            P=Vec3(0,0,0);
            L=Vec3(0,0,0);
            E=0;
            Erot=0;
            Ekin=0;
            Epot=0;
            //!creating a graphical interface
            char b[120];
            sprintf(b,"poli.pov%05d.pov",j/paso);
            CGraph gp(b);
            //!set position of the camera
	    r=17*J+3*K+100*t*K;
            r=17*(sin(0.02*M_PI*(t+4))*I-cos(0.02*M_PI*(t+4))*J)+(10-0.2*t)*K;
            //!set aim of the camera
	    F=-5*K; 
	    gp.Set_Camera(r,F);
            //!here we move the hopper according to the displacement vector v
            if(j/paso==200) {
                //v=-2.*K;
                v=-20.*K;
                wall.moveBy(v);
            }
            //!for all the particles 
            for (i=0;i<N;i++) {
                //!plot particle 
                gp.Graph_Particle(Particles[i],"Gray");
                //!calculate energy variables
                P+=Particles[i].getp();
                L+=Particles[i].getLs();
                Ekin+=Particles[i].getEkin();
                Erot+=Particles[i].getErot();
                E+=Particles[i].getEkin()+Particles[i].getErot();
            }
            //!plot walls            
            gp.Graph_Particle(wall,"Green"); 
            //gp.Graph_Particle(cow,"Gray"); 
            ///!gp.Graph_Particle(pirinola,"Blue");

            //!for all particle interaction
            for(it=Inaron.begin();it!=Inaron.end();it++) {
                //gp.Graph_Full_contact((*it).second,0.05,"Blue");
                //!calculate potential energy
                Epot+=(*it).second.getEpot();
                E+=(*it).second.getEpot();
            }
            //!close graphical interface (file)
            gp.close();
            //!make a plot 
            gp.Animation(j/paso);
            //Pf.precision(15);
            Pf << t<<" "<<P.X()<<" "<<P.Y()<<" "<<P.Z()<<" "<<P.norm()<<endl;
            Lf << t<<" "<<L.X()<<" "<<L.Y()<<" "<<L.Z()<<" "<<L.norm()<<endl;
            Ef << t<<" "<<Ekin<<" "<<Erot<<" "<<Epot<<" "<<E<<endl;
            Ff << t<<" "<<np<<endl;
        }
    }
    Pf.close();
    Lf.close();
    Ef.close();
    Ff.close();
    return 0;
    
}
