#include "Interacton.h"

CInteracton::CInteracton(CParticle & p1,CParticle & p2)
{
    m_P1=&p1;
    m_P2=&p2;
    m_kn=10000.;
    m_kt=5000.;
    m_gn=16;
    m_gt=8;
    m_mu=0.4;
    //m_gn=0;
    //m_gt=0;
    m_Epot=0;
    m_c=false;
}


void CInteracton::calcForce(double dt) {
    double delta,dist,alpha;
    CEdge3d F;
    Vec3 f,ft,n,r,T,vrel,vt,t;
    Quaternion q;

    //!get radius
    double & R1=(m_P1->m_R);
    double & R2=(m_P2->m_R);

    //!get number of vertices, edges and faces
    int & Nv1= m_P1->m_Nv;
    int & Nv2= m_P2->m_Nv;

    int & Ne1= m_P1->m_Ne;
    int & Ne2= m_P2->m_Ne;

    int & Nf1= m_P1->m_Nf;
    int & Nf2= m_P2->m_Nf;

    //!get the center of masses 
    Vec3 & r1 = m_P1->m_r;
    Vec3 & r2 = m_P2->m_r;

    //!get linear velocities
    Vec3 & v1 = m_P1->m_v;
    Vec3 & v2 = m_P2->m_v;

    //!get linear velocities
    Vec3 & ome1 = m_P1->m_ome;
    Vec3 & ome2 = m_P2->m_ome;

    //!get radious of the enclosing spheres
    double dmax1 = m_P1->m_dmax;
    double dmax2 = m_P2->m_dmax;
    
    //!get quaternions
    Quaternion Q1 =  m_P1->m_Q;
    Quaternion Q2 =  m_P2->m_Q;

    //! get forces 
    Vec3 & Force1 = m_P1->m_f;
    Vec3 & Force2 = m_P2->m_f;

    //! get torques 
    Vec3 & Torque1 = m_P1->m_T;
    Vec3 & Torque2 = m_P2->m_T;

    int i,j;

    //!init potential energy
    m_Epot=0;

    //!init active contact flag
    m_c=false;
    
    //!init indexes of features 
    pair<int,int> p;
    
    //!init normal force 
    m_fn=Vec3(0,0,0);

    //!distance between center of masses 
    r=r1-r2;
    dist=r.norm();

    //!relative linear velocity
    vrel=v2-v1;

    //!sum of radii between the enclosing spheres 
    delta=dmax1 + dmax2;
    

    //!neighbour check: if the distance between the center of mass is lower that the sum of radii of enclosing spheres
    if(dist<=delta) {
        //!for all edges of first particle 
        for (i=0;i<Ne1;i++) {
            //!for all edges of second particle 
            for (j=0;j<Ne2;j++) {
                //!calculate edge-edge distance 
                //cout<<"edge-edge "<<i<<" "<<j<<endl;
                //!cout<<Nd1<<
                F=distance_Edge3d_Edge3d(m_P1->m_Edge[i],m_P2->m_Edge[j]);
                dist=F.DX().norm();
                delta=R1+R2-dist;
                //!if overlapping length is larger than o
                if (delta>0) {
                    //!active contact
                    m_c=true;
                    //!get unit vector
                    n=F.DX().unit();
                    //!normal elastic force 
                    f=n*(m_kn*delta);
                    m_fn+=f;
                    //!point of application of the contact force 
                    r=F.I()+n*((R1*R1-R2*R2+dist*dist)/(2*dist));              
                    //!alpha is chosen depending on Cundall model (1) and MnNamara correction (R1+R2)/dist;
                    alpha=1;
                    //alpha=(R1+R2)/dist;                    
                    //!objective calculation of relative velocity 
                    vrel=-(alpha*(v2-v1)+cross(rotate(Q2,ome2),r-r2)-cross(rotate(Q1,ome1),r-r1));
                    //!tangential relative velocity
                    vt=vrel-dot(n,vrel)*n;
                    //!indexes of the particles 
                    p=make_pair(i,j);
                    //!update tangential spring
                    m_free[p]+=vt*dt;
                    m_free[p]-=dot(m_free[p],n)*n;
                    t=m_free[p].unit();
                    if(m_free[p].norm()>m_mu*f.norm()/m_kt) {
                        m_free[p]=t*m_mu*f.norm()/m_kt;
                    }
                    ft=m_kt*m_free[p];
                    //!update potential energy
                    m_Epot+=0.5*m_kn*delta*delta+0.5*m_kt*m_free[p]*m_free[p];
                    
                    //!update forces
                    Force1 -= f+m_gn*dot(n,vrel)*n+m_gt*vt+ft;
                    Force2 += f+m_gn*dot(n,vrel)*n+m_gt*vt+ft;
                
                    //!update torque of first particle
                    T=cross(r-r1,f+m_gn*dot(n,vrel)*n+m_gt*vt+ft);
                    q=Q1.conj();
                    Torque1 -=rotate(q,T);

                    //!update torque of second particle 
                    T=cross(r-r2,f+m_gn*dot(n,vrel)*n+m_gt*vt+ft);
                    q=Q2.conj();
                    Torque2 +=rotate(q,T);
                
                }
            }
        }
        //!for all vertices of the first particle      
        for (i=0;i<Nv1;i++) {
            //!for all faces of the second particle 
            for (j=0;j<Nf2;j++) {
                //cout<<"vertex-face "<<i<<" "<<j<<endl;
                //!calculate distance vertice to Face
                F=distance_point_Face3d(m_P1->m_ve[i],m_P2->m_Face[j]);
                dist=F.DX().norm();
                delta=R1+R2-dist;
                //!if overlapping length is larger than zero
                if (delta>0) {
                    //!activate contact
                    m_c=true;
                    //!get unit normal
                    n=F.DX().unit();
                    //!calculate normal elastic force 
                    f=n*(m_kn*delta);
                    m_fn+=f;
                    //!calculate point of application of the force 
                    r=F.I()+n*((R1*R1-R2*R2+dist*dist)/(2*dist));
                    
                    //!alpha is chosen depending on Cundall model (1) and MnNamara correction (R1+R2)/dist;
                    alpha=1;
                    //alpha=(R1+R2)/dist;

                    //!objective relative tangential velocity
                    vrel=-(alpha*(v2-v1)+cross(rotate(Q2,ome2),r-r2)-cross(rotate(Q1,ome1),r-r1));
                    vt=vrel-dot(n,vrel)*n;

                    //!indexes of the pair
                    p=make_pair(i,j);
                    
                    //!update tangential elastic spring
                    m_frvf[p]+=vt*dt;
                    m_frvf[p]-=dot(m_frvf[p],n)*n;
                    t=m_frvf[p].unit();
                    if(m_frvf[p].norm()>m_mu*f.norm()/m_kt) {
                        m_frvf[p]=t*m_mu*f.norm()/m_kt;
                    }
                    ft=m_kt*m_frvf[p];
                    
                    //!potential energy
                    m_Epot+=0.5*m_kn*delta*delta+0.5*m_kt*m_frvf[p]*m_frvf[p];
                    
                    //!update forces 
                    Force1-=f+m_gn*dot(n,vrel)*n+m_gt*vt+ft;
                    Force2+=f+m_gn*dot(n,vrel)*n+m_gt*vt+ft;

                    //!update torques
                    T=cross(r-r1,f+m_gn*dot(n,vrel)*n+m_gt*vt+ft);
                    q=Q1.conj();
                    Torque1-=rotate(q,T);

                    T=cross(r-r2,f+m_gn*dot(n,vrel)*n+m_gt*vt+ft);
                    q=Q2.conj();
                    Torque2+=rotate(q,T);

                }
            }
        }
    
        //!for all faces of the first particle 
        for (i=0;i<Nf1;i++) {
            //!for all vertices of the second particle 
            for (j=0;j<Nv2;j++) {
                //cout<<"face-vertex "<<i<<" "<<j<<endl;
                //!calculate vertex-face distance
                F=distance_point_Face3d(m_P2->m_ve[j],m_P1->m_Face[i]);
                F.invert();
                dist=F.DX().norm();
                delta=R1+R2-dist;
                //!if overlapping lenght is larger than zero
                if (delta>0) {
                    //!activate contact
                    m_c=true;
                    //!normal force 
                    n=F.DX().unit();
                    //!elastic part of the normal force 
                    f=n*(m_kn*delta);
                    m_fn+=f;
                    //!point of application of the force 
                    r=F.I()+n*((R1*R1-R2*R2+dist*dist)/(2*dist));

                    //!alpha is chosen depending on Cundall model (1) and MnNamara correction (R1+R2)/dist;
                    alpha=1;
                    //alpha=(R1+R2)/dist;
                    
                    //!objective relative velocity at the contact
                    vrel=-(alpha*(v2-v1)+cross(rotate(Q2,ome2),r-r2)-cross(rotate(Q1,ome1),r-r1));
                    //!relative tangential velocity
                    vt=vrel-dot(n,vrel)*n;
                    //!indexes of the features 
                    p=make_pair(i,j);

                    //!update elastic tangential stiffnesses
                    m_frfv[p]+=vt*dt;
                    m_frfv[p]-=dot(m_frfv[p],n)*n;
                    t=m_frfv[p].unit();
                    if(m_frfv[p].norm()>m_mu*f.norm()/m_kt) {
                        m_frfv[p]=t*m_mu*f.norm()/m_kt;
                    }
                    ft=m_kt*m_frfv[p];
                    
                    //!potential energy
                    m_Epot+=0.5*m_kn*delta*delta+0.5*m_kt*m_frfv[p]*m_frfv[p];

                    //!update forces 
                    Force1-=f+m_gn*dot(n,vrel)*n+m_gt*vt+ft;
                    Force2+=f+m_gn*dot(n,vrel)*n+m_gt*vt+ft;

                    //!update torques
                    T=cross(r-r1,f+m_gn*dot(n,vrel)*n+m_gt*vt+ft);
                    q=Q1.conj();
                    Torque1-=rotate(q,T);

                    T=cross(r-r2,f+m_gn*dot(n,vrel)*n+m_gt*vt+ft);
                    q=Q2.conj();
                    Torque2+=rotate(q,T);

                }

            }
        }
    }    
}
