#include "Particle.h"

CParticle::CParticle(const string & filename_particle)
{
  //!create stream 
  ifstream myfile ( filename_particle.c_str() );
  cout<<"Particle:reading "<<filename_particle<<endl;
  Vec3 Vertex, V0, V1;
  int NumVertices;
  vector<Vec3> VerticesList;

  //!if imput file is open
  if (!myfile.is_open())
  {
    cout << "CParticle::CParticle: Unable to open file " <<filename_particle<<endl;
    exit(1);
  }
  //!-read sphero-radius
  myfile >> m_R;
  cout<<"spheroradius ="<< m_R <<endl;
  //!-read number of vertices 
  myfile >> m_Nv;
  cout <<"number of vertices = "<< m_Nv << endl;
  //!-create vector container for the vertices 
  m_ve.reserve(m_Nv);
  //!-read the vertices 
  for (int i=0;i<m_Nv;++i)
  {
     myfile >> Vertex;
     cout << Vertex << endl;
     m_ve.push_back(Vertex);
  }
  //!-read the edges
   myfile >> m_Ne;
   cout <<"number of edges = "<<m_Ne<<endl;
   m_Edge.reserve(m_Ne);
   for (int i=0; i<m_Ne;++i)
   {
      myfile >> V0 >> V1;
      cout<< V0<<" "<<V1<<endl;
      m_Edge.push_back(CEdge3d(V0,V1));
   } 
   //!-read the faces
   myfile >> m_Nf;
   cout<<"number of faces = "<<m_Nf<<endl;
   m_Face.reserve(m_Nf);
   for (int i=0; i<m_Nf; ++i)
   {
      myfile >> NumVertices;
      cout<<NumVertices<<endl;
      VerticesList.reserve(NumVertices);
      for (int j=0; j<NumVertices;++j)
      {
         myfile >> Vertex;
         cout<<Vertex<<endl;
         VerticesList.push_back(Vertex);
       }
       m_Face.push_back(CFace3d(VerticesList));
       VerticesList.clear();
   }

   m_Faceold = m_Face;
   m_Edgeold = m_Edge;
   m_veold   = m_ve;

  m_m  = 1.0;
  //m_R  = 1.0;
  m_Ixx= 1.0;
  m_Iyy= 1.0;
  m_Izz= 1.0;

  m_Ekin = 0.0;
  m_Erot = 0.0;
  m_dmax = 1.0; //temporal

  Vec3 Z = Vec3(0,0,0);
  m_r = Z;
  m_v = Z;
  m_f = Z;
  m_T = Z;
  m_Td= Z;
  m_ome=Z;
  m_fome=Z;
  m_r1=0;
  m_r2=0;
  Quaternion Q0(1,Z);
  m_Q= Q0;
  m_large = false;
  cout<<"starting calculating mass properties"<<endl;
  calcMassProperties(10);
  cout<<"finishing calculating mass properties"<<endl;
}


CParticle::CParticle( const vector<Vec3>           & VerticesList,
                   const vector< pair<Vec3,Vec3> > & EdgesList, 
                   const vector< vector<Vec3> >    & FacesList   )
{
   //!get the number of edges, faces and vertices
   m_Nf = FacesList.size();
   m_Ne = EdgesList.size();
   m_Nv = VerticesList.size();

   //!reserve space of the memory for the vertices, edges and fases
   m_ve.reserve(m_Nv); 
   m_veold.reserve(m_Nv);
   m_Edge.reserve(m_Ne);
   m_Edgeold.reserve(m_Ne);
   m_Face.reserve(m_Nf);
   m_Faceold.reserve(m_Nf);

   //!iterators
   vector<Vec3>::const_iterator itv;
   //vector<CEdge3d>::iterator ite;
   //vector<CFace3d>::iterator itf;
   vector<pair<Vec3,Vec3> >::const_iterator ite;
   vector< vector<Vec3> >::const_iterator itf;
   
   //!setting up vertices
   m_ve = VerticesList;
   m_veold = m_ve;
   
   pair<Vec3,Vec3> pair;
   //!setting up edges
   for (ite = EdgesList.begin();ite!=EdgesList.end();++ite)
   {
       pair = *ite;
       m_Edge.push_back( CEdge3d( pair.first,pair.second) );
   }
   m_Edgeold = m_Edge;

   //!setting up edges
   for(itf = FacesList.begin();itf!=FacesList.end();++itf)
   {
       m_Face.push_back( CFace3d(*itf) );
   }
  m_Faceold = m_Face;

  m_m  = 1.0;
  m_R  = 1.0;
  m_Ixx= 1.0;
  m_Iyy= 1.0;
  m_Izz= 1.0;

  m_Ekin = 0.0;
  m_Erot = 0.0;
  m_dmax = 1.0; //temporal

  Vec3 Z = Vec3(0,0,0);
  m_r = Z;
  m_v = Z;
  m_f = Z;
  m_T = Z;
  m_Td= Z;
  m_ome=Z;
  m_fome=Z;
  m_r1=0;
  m_r2=0;
  Quaternion Q0(1,Z);
  m_Q= Q0;
  m_large = true;
}

//!\todo reserve elements
CParticle::CParticle(double l0, double m0, double R0,Vec3 &r0, Vec3 &v0,  Vec3 &O,Quaternion &Q0,string p) {
    double wx,wy,wz;
    int i,j,n;
    Vec3 zero=Vec3(0,0,0);
    CFace3d Face;
    CEdge3d Edge;
    //cout << m_Face<< " " <<m_Faceold<< endl;
    m_large=false;
    //if(p=="tetra") {
    vector<CFace3d>::iterator itf;
    vector<CEdge3d>::iterator ite;
    vector<Vec3>::iterator itv;
    Vec3 tempv;
   

    if(p=="yermis") {
        m_dmax=l0/2+R0;
        m_Nf=0;
        m_Ne=3;
        m_Nv=6;
        m_Q=Q0.conj();
        m_r=r0;
        //m_r.show_on_screen(); cout <<endl;
        m_v=v0;
        m_ome=m_fome=rotate(m_Q,O);
        m_Q=Q0;
        m_T=Vec3(0,0,0);
        m_f=Vec3(0,0,0);
        m_R=R0;
        m_m=3*((4./3.)*M_PI*m0*R0*R0*R0+m0*M_PI*R0*R0*l0);
        
        tempv=Vec3(0,0,-l0/2);
        itv=m_veold.end();
        m_veold.insert(itv,tempv);
        
        tempv=Vec3(0,0,l0/2);
        itv=m_veold.end();
        m_veold.insert(itv,tempv);
        
        tempv=Vec3(0,-l0/2,0);
        itv=m_veold.end();
        m_veold.insert(itv,tempv);
        
        tempv=Vec3(0,l0/2,0);
        itv=m_veold.end();
        m_veold.insert(itv,tempv);
        
        tempv=Vec3(-l0/2,0,0);
        itv=m_veold.end();
        m_veold.insert(itv,tempv);
        
        tempv=Vec3(l0/2,0,0);
        itv=m_veold.end();
        m_veold.insert(itv,tempv);
        
        for(i=0;i<m_Nv;i++) {
            itv=m_ve.end();
            m_ve.insert(itv,rotate(m_Q,m_veold[i])+m_r);
        }
        
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[0],m_veold[1]));
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[2],m_veold[3]));
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[4],m_veold[5]));
        
        for(i=0;i<m_Ne;i++) {
            m_Edgeold[i].translate(m_r);
            ite=m_Edge.end();
            m_Edge.insert(ite,rotation_edge(m_Edgeold[i],m_Q,m_r));
        }
        m_Izz=m_Ixx=m_Iyy=(2./5.)*M_PI*m_R*m_R*l0*m0*m_R*m_R+0.5*(4*M_PI/3)*m_R*m_R*m_R*m0*m_R*m_R+(1./12.)*M_PI*m_R*m_R*l0*m0*(3*m_R*m_R+l0*l0)+(2./5.)*(4*M_PI/3)*m_R*m_R*m_R*m0*m_R*m_R+(4*M_PI/3)*m_R*m_R*m_R*m0*0.25*l0*l0;
       
        wx=m_ome.X();
        wy=m_ome.Y();
        wz=m_ome.Z();

        m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
        m_Ekin=0.5*m_m*dot(m_v,m_v);
    }
    
        
    if(p=="rice") {
        m_dmax=l0/2+R0;
        m_Nf=0;
        m_Ne=1;
        m_Nv=2;
        m_Q=Q0.conj();
        m_r=r0;
        //m_r.show_on_screen(); cout <<endl;
        m_v=v0;
        m_ome=m_fome=rotate(m_Q,O);
        m_Q=Q0;
        m_T=Vec3(0,0,0);
        m_f=Vec3(0,0,0);
        m_R=R0;
        m_m=(4./3.)*M_PI*m0*R0*R0*R0+m0*M_PI*R0*R0*l0;

        tempv=Vec3(0,0,-l0/2);
        m_veold.push_back(tempv);
        m_ve.push_back(tempv);
      
        tempv=Vec3(0,0,l0/2);
        m_veold.push_back(tempv);
        m_ve.push_back(tempv);

        m_Edgeold.push_back(CEdge3d(m_veold[0],m_veold[1]));
        m_Edge.push_back(CEdge3d(m_veold[0],m_veold[1]));

        m_Izz=(2./5.+0.5)*m_m*m_R*m_R;
        m_Ixx=m_Iyy=(2./5.)*m_m*m_R*m_R+0.25*m_m*l0*l0+(1./12.)*m_m*(3*m_R*m_R+l0*l0);

        wx=m_ome.X();
        wy=m_ome.Y();
        wz=m_ome.Z();

        m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
        m_Ekin=0.5*m_m*dot(m_v,m_v);
    }
    
    if(p=="tetra") {

        //!maximal diameter
        m_dmax=sqrt(3./8.)*l0+R0;

        //!number of faces, edges and vertices
        m_Nf=4;
        m_Ne=6;
        m_Nv=4;

        Vec3 v[3];
        //m_Face=new CFace3d[m_Nf];
        //m_Faceold=new CFace3d[m_Nf];
        //cout << m_Face<< " " <<m_Faceold<< endl;
        m_Q=Q0.conj();
        m_r=r0;
        //m_r.show_on_screen(); cout <<endl;
        m_v=v0;
        m_ome=m_fome=rotate(m_Q,O);
        m_Q=Q0;
        m_T=Vec3(0,0,0);
        m_f=Vec3(0,0,0);
        m_R=R0;
        m_m=l0*l0*l0*sqrt(2)/12.;

        //!creating the faces 
        m_Faceold.reserve(4);

        v[0]=Vec3(l0/2,0,l0/sqrt(8));
        v[1]=Vec3(-l0/2,0,l0/sqrt(8));
        v[2]=Vec3(0,-l0/2,-l0/sqrt(8));
        m_Faceold.push_back(CFace3d(v,3));
       
        v[0]=Vec3(-l0/2,0,l0/sqrt(8));
        v[1]=Vec3(l0/2,0,l0/sqrt(8));
        v[2]=Vec3(0,l0/2,-l0/sqrt(8));
        m_Faceold.push_back(CFace3d(v,3));

        v[0]=Vec3(-l0/2,0,l0/sqrt(8));
        v[1]=Vec3(0,l0/2,-l0/sqrt(8));
        v[2]=Vec3(0,-l0/2,-l0/sqrt(8));
        m_Faceold.push_back(CFace3d(v,3));

        v[0]=Vec3(l0/2,0,l0/sqrt(8));
        v[1]=Vec3(0,-l0/2,-l0/sqrt(8));
        v[2]=Vec3(0,l0/2,-l0/sqrt(8));
        m_Faceold.push_back(CFace3d(v,3));
        
        
        //!faces in the current position (\todo)
        for(i=0;i<m_Nf;i++) {
            Face = rotation_face(m_Faceold[i],m_Q,zero);
            Face.translation( m_r);
            m_Face.push_back(Face);
        }
        
        //!creating the vertices
        m_veold.reserve(4);
        m_veold.push_back(Vec3(l0/2,0,l0/sqrt(8)));
        m_veold.push_back(Vec3(-l0/2,0,l0/sqrt(8)));
        m_veold.push_back(Vec3(0,-l0/2,-l0/sqrt(8)));
        m_veold.push_back(Vec3(0,l0/2,-l0/sqrt(8)));
        
        //!vertices in the current position (\todo)
        for(i=0;i<m_Nv;i++) {
            itv=m_ve.end();
            m_ve.insert(itv,rotate(m_Q,m_veold[i])+m_r);
        }

        //!creating edges
        m_Edgeold.reserve(6);
        for(i=0;i<m_Nv-1;i++) {
            for(j=i+1;j<m_Nv;j++) {
                m_Edgeold.push_back(CEdge3d(m_veold[i],m_veold[j]));
            }
        }

        //!edges in the current position (\todo)        
        for(i=0;i<m_Ne;i++) {
           Edge = rotation_edge(m_Edgeold[i],m_Q,zero);
           Edge.translate(m_r);
           m_Edge.push_back(Edge);
        }

        //!moment of inertia
        m_Ixx=m_Iyy=m_Izz=m_m*l0*l0/20;

        //!angular momentum
        wx=m_ome.X();
        wy=m_ome.Y();
        wz=m_ome.Z();

	//!kinetic energy
        m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
        m_Ekin=0.5*m_m*dot(m_v,m_v);

    }
    
    if (p=="tippetop") {
        m_dmax=2*R0;
        m_dmax=sqrt(3./8.)*l0+R0;
        m_Nf=0;
        m_Ne=0;
        m_Nv=1;
        m_Q=Q0.conj();
        m_r=r0;
        m_v=v0;
        m_ome=m_fome=rotate(m_Q,O);
        m_Q=Q0;
        m_T=Vec3(0,0,0);
        m_f=Vec3(0,0,0);
        m_R=R0;
        m_m=m0;
        
        itv=m_veold.end();
        m_veold.insert(itv,Vec3(0,0,l0));
        
        itv=m_ve.end();
        m_ve.insert(itv,rotate(m_Q,m_veold[0])+m_r);
        
    
        m_Ixx=m_Iyy=m_Izz=(2./5.)*m_m*m_R*m_R;

        wx=m_ome.X();
        wy=m_ome.Y();
        wz=m_ome.Z();

        m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
        m_Ekin=0.5*m_m*dot(m_v,m_v);
        
        
    }
    
    if (p=="pirinola") {
        m_dmax=2*l0;
        Vec3 v[6];
        m_Nf=13;
        m_Ne=24;
        m_Nv=13;
        m_Q=Q0.conj();
        m_r=r0;
        m_v=v0;
        m_ome=m_fome=O;
        m_Q=Q0;
        m_T=Vec3(0,0,0);
        m_f=Vec3(0,0,0);
        m_R=R0;
        m_m=m0;

        itv=m_veold.end();
        m_veold.insert(itv,Vec3(0,0,-l0));
        for (i=0;i<6;i++) {
            itv=m_veold.end();
            m_veold.insert(itv,Vec3(l0*cos(M_PI*i/3)/2,l0*sin(M_PI*i/3)/2,-l0/2));
            itv=m_veold.end();
            m_veold.insert(itv,Vec3(l0*cos(M_PI*i/3)/2,l0*sin(M_PI*i/3)/2,l0/2));
            v[0]=Vec3(l0*cos(M_PI*i/3)/2,l0*sin(M_PI*i/3)/2,-l0/2);
            v[1]=Vec3(l0*cos(M_PI*i/3)/2,l0*sin(M_PI*i/3)/2,l0/2);
            v[2]=Vec3(l0*cos(M_PI*(i+1)/3)/2,l0*sin(M_PI*(i+1)/3)/2,l0/2);
            v[3]=Vec3(l0*cos(M_PI*(i+1)/3)/2,l0*sin(M_PI*(i+1)/3)/2,-l0/2);
            ite=m_Edgeold.end();
            m_Edgeold.insert(ite,CEdge3d(v[0],v[1]));
            ite=m_Edgeold.end();
            m_Edgeold.insert(ite,CEdge3d(v[0],v[3]));
            ite=m_Edgeold.end();
            m_Edgeold.insert(ite,CEdge3d(v[1],v[2]));
            itf=m_Faceold.end();
            m_Faceold.insert(itf,CFace3d(v,4));
            v[0]=Vec3(l0*cos(M_PI*i/3)/2,l0*sin(M_PI*i/3)/2,-l0/2);
            v[1]=Vec3(l0*cos(M_PI*(i+1)/3)/2,l0*sin(M_PI*(i+1)/3)/2,-l0/2);
            v[2]=Vec3(0,0,-l0);
            ite=m_Edgeold.end();
            m_Edgeold.insert(ite,CEdge3d(v[0],v[2]));
            itf=m_Faceold.end();
            m_Faceold.insert(itf,CFace3d(v,3));
        }
        for (i=0;i<6;i++) {
            v[i]=Vec3(l0*cos(M_PI*i/3)/2,l0*sin(M_PI*i/3)/2,l0/2);
        }
        itf=m_Faceold.end();
        m_Faceold.insert(itf,CFace3d(v,6));
        
        for(i=0;i<m_Ne;i++) {
            m_Edgeold[i].translate(m_r);
            ite=m_Edge.end();
            m_Edge.insert(ite,rotation_edge(m_Edgeold[i],m_Q,m_r));
        }
        
        for(i=0;i<m_Nf;i++) {
            m_Faceold[i].translation(m_r);
            itf=m_Face.end();
            m_Face.insert(itf,rotation_face(m_Faceold[i],m_Q,m_r));
        }
        
        for(i=0;i<m_Nv;i++) {
            itv=m_ve.end();
            m_ve.insert(itv,rotate(m_Q,m_veold[i])+m_r);
        }
        
        
        

        m_Ixx=m_Iyy=(1./12.)*m_m*(3*(l0/2)*(l0/2)+l0*l0);
        m_Izz=0.125*m_m*l0*l0;
        wx=m_ome.X();
        wy=m_ome.Y();
        wz=m_ome.Z();

        m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
        m_Ekin=0.5*m_m*dot(m_v,m_v);


    }


}

//!corrected
CParticle::CParticle(double a,double b,double R0,Vec3 &r0,Quaternion &Q0, const Vec3 & v0) {
    double wx,wy,wz;
    vector<CFace3d>::iterator itf;
    vector<CEdge3d>::iterator ite;
    vector<Vec3>::iterator itv;
    m_Nf=1;
    m_Ne=4;
    m_Nv=4;
    Vec3 v[4];
    int i;
    m_r=r0;
    m_R=R0;
    m_Q=Q0;
    m_large=true;
    m_m=1;
    m_dmax=sqrt((a/2)*(a/2)+(b/2)*(b/2))+R0;
    m_v=v0;
    m_T=Vec3(0,0,0);
    m_f=Vec3(0,0,0);
    m_ome=m_fome=Vec3(0,0,0);    

    //!list of vertices
    v[0]=Vec3(a/2,b/2,0);
    v[1]=Vec3(-a/2,b/2,0);
    v[2]=Vec3(-a/2,-b/2,0);
    v[3]=Vec3(a/2,-b/2,0);

    //!vertices
    for(i=0;i<m_Nv;i++) {
        m_veold.push_back(v[i]);
        m_ve.push_back(v[i]);
    }

    //!edges
    for(i=0;i<m_Nv;i++) {
        m_Edgeold.push_back(CEdge3d(m_veold[i],m_veold[(i+1)%m_Nv]));
        m_Edge.push_back(CEdge3d(m_veold[i],m_veold[(i+1)%m_Nv]));
    }

    //!face
    m_Faceold.push_back(CFace3d(v,4));
    m_Face.push_back(CFace3d(v,4));
   
    updateVertices();

    wx=m_ome.X();
    wy=m_ome.Y();
    wz=m_ome.Z();
    
    m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
    m_Ekin=0.5*m_m*dot(m_v,m_v);
}

CParticle::CParticle(double a,double b,double c,double R0,Vec3 &r0,Quaternion &Q0) {
    double wx,wy,wz;
    vector<CFace3d>::iterator itf;
    vector<CEdge3d>::iterator ite;
    vector<Vec3>::iterator itv;
    Vec3 v[4];
    int i,j;
    m_Nf=4;
    m_Ne=12;
    m_Nv=8;
    m_r=r0;
    m_R=R0;
    //cout << m_R<< endl;
    m_Q=Q0;
    m_large=true;
    m_dmax=sqrt((a/2)*(a/2)+(c/2)*(c/2))+R0;
    m_v=Vec3(0,0,0);
    m_ome=m_fome=Vec3(0,0,0);
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(a/2,a/2,c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(a/2,-a/2,c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(-a/2,-a/2,c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(-a/2,a/2,c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(b/2,b/2,-c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(b/2,-b/2,-c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(-b/2,-b/2,-c/2));
    
    itv=m_veold.end();
    m_veold.insert(itv,Vec3(-b/2,b/2,-c/2));
    
    for(i=0;i<m_Nv;i++) {
        itv=m_ve.end();
        m_ve.insert(itv,rotate(m_Q,m_veold[i])+m_r);
    }
    
    v[0]=Vec3(a/2,a/2,c/2);
    v[1]=Vec3(b/2,b/2,-c/2);
    v[2]=Vec3(b/2,-b/2,-c/2);
    v[3]=Vec3(a/2,-a/2,c/2);
    itf=m_Faceold.end();
    m_Faceold.insert(itf,CFace3d(v,4));

    v[0]=Vec3(-a/2,-a/2,c/2);
    v[1]=Vec3(-b/2,-b/2,-c/2);
    v[2]=Vec3(b/2,-b/2,-c/2);
    v[3]=Vec3(a/2,-a/2,c/2);
    itf=m_Faceold.end();
    m_Faceold.insert(itf,CFace3d(v,4));

    v[0]=Vec3(-a/2,a/2,c/2);
    v[1]=Vec3(-b/2,b/2,-c/2);
    v[2]=Vec3(-b/2,-b/2,-c/2);
    v[3]=Vec3(-a/2,-a/2,c/2);
    itf=m_Faceold.end();
    m_Faceold.insert(itf,CFace3d(v,4));

    v[0]=Vec3(-a/2,a/2,c/2);
    v[1]=Vec3(-b/2,b/2,-c/2);
    v[2]=Vec3(b/2,b/2,-c/2);
    v[3]=Vec3(a/2,a/2,c/2);
    itf=m_Faceold.end();
    m_Faceold.insert(itf,CFace3d(v,4));

    for(i=0;i<4;i++) {
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[i],m_veold[(i+1)%4]));
    }

    for(i=0;i<4;i++) {
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[i+4],m_veold[(i+1)%4+4]));
    }

    for(i=0;i<4;i++) {
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[i],m_veold[i+4]));
    }

    for(i=0;i<m_Nf;i++) {
        m_Faceold[i].translation(m_r);
    }
    for(i=0;i<m_Nf;i++) {
        itf=m_Face.end();
        m_Face.insert(itf,rotation_face(m_Faceold[i],m_Q,m_r));
    }
    for(i=0;i<m_Nv;i++) {
        itv=m_ve.end();
        m_ve.insert(itv,rotate(m_Q,m_veold[i])+m_r);
    }

    for(i=0;i<m_Ne;i++) {
        ite=m_Edge.end();
        m_Edgeold[i].translate(m_r);
        m_Edge.insert(ite,rotation_edge(m_Edgeold[i],m_Q,m_r));
    }
    
    wx=m_ome.X();
    wy=m_ome.Y();
    wz=m_ome.Z();
    
    m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);
    m_Ekin=0.5*m_m*dot(m_v,m_v);

}

CParticle::CParticle(double a,double b,int N,double R0,double l0,Vec3 &r0,Vec3 &v0,Quaternion &Q0) {
    srand(100);
    Vec3 rab,rar,r,d,I(1,0,0),J(0,1,0),K(0,0,1),Z(0,0,0);
    vector<CFace3d>::iterator itf;
    vector<CEdge3d>::iterator ite;
    vector<Vec3>::iterator itv;
    Vec3 v[4];
    int i,j;
    m_Nf=1;
    m_Ne=N+4;
    m_Nv=N+4;
    m_r=r0;
    m_R=R0;
    m_Q=Q0;
    m_large=false;
    m_dmax=sqrt((a/2)*(a/2)+(b/2)*(b/2))+R0;
    m_v=v0;
    m_ome=m_fome=Vec3(0,0,0);
    m_m=1;
    v[0]=Vec3(a/2,b/2,0);
    v[1]=Vec3(-a/2,b/2,0);
    v[2]=Vec3(-a/2,-b/2,0);
    v[3]=Vec3(a/2,-b/2,0);
    itf=m_Faceold.end();
    m_Faceold.insert(itf,CFace3d(v,4));
    i=0;
    m_Faceold[i].translation(m_r);
    itf=m_Face.end();
    m_Face.insert(itf,rotation_face(m_Faceold[i],m_Q,m_r));
    for(i=0;i<4;i++) {
        itv=m_veold.end();
        m_veold.insert(itv,v[i]);
    }
    for(i=0;i<4;i++) {
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(m_veold[i],m_veold[(i+1)%4]));
    }
    
    
    for(i=0;i<N;i++) {
        r=(a*rand()/RAND_MAX-a/2)*I+(b*rand()/RAND_MAX-b/2)*J;
        rab=r;
        rar=r+0.01*rand()/RAND_MAX*I+0.01*rand()/RAND_MAX*J+l0*K;
        ite=m_Edgeold.end();
        m_Edgeold.insert(ite,CEdge3d(rab,rar));
        itv=m_veold.end();
        m_veold.insert(itv,rar);
    }
    
    for(i=0;i<m_Nv;i++) {
        itv=m_ve.end();
        m_ve.insert(itv,rotate(m_Q,m_veold[i])+m_r);
    }
    
    for(i=0;i<m_Ne;i++) {
        m_Edgeold[i].translate(m_r);
        ite=m_Edge.end();
        m_Edge.insert(ite,rotation_edge(m_Edgeold[i],m_Q,m_r));
    } 
}


void CParticle::calcMassProperties(const int N)
{
  double   V = 0;       //!volumen of  of the particle
  Vec3    rm = Vec3(0); //!center of mass
  double Ixx = 0;       //!temporal moment of inertia Ixx,Iyy,Izz
  Vec3 Point;           //!test point
  bool b=false;         //!bool variable to check whether point is inside

  //!for calculation of the bounding box
  double Xmax=m_ve[0].X();
  double Xmin=m_ve[0].X();
  double Ymax=m_ve[0].Y();
  double Ymin=m_ve[0].Y();
  double Zmax=m_ve[0].Z();
  double Zmin=m_ve[0].Z();
   
  vector<Vec3>::iterator it;
  for (it=m_ve.begin();it!=m_ve.end();++it)
  { 
    Xmax = max(Xmax,it->X());
    Xmin = min(Xmin,it->X());
    Ymax = max(Ymax,it->Y());
    Ymin = min(Ymin,it->Y()); 
    Zmax = max(Ymax,it->Z());
    Zmin = min(Ymin,it->Z()); 
  }
  
  //!montecarlo calculation of the mass properties
  double dx=(Xmax-Xmin)/N;
  double dy=(Ymax-Ymin)/N;
  double dz=(Zmax-Zmin)/N;

  double dV=dx*dy*dz;
  
  for (double x = Xmin; x < Xmax; x+=dx){
    for (double y = Ymin; y < Ymax; y+=dy){
      for (double z = Zmin; z < Zmax; z+=dz){
         Point=Vec3(x,y,z);
         b=this->pointIsInsideParticle(Point); //todo
         if (b==true)
         {
            V  += 1;
            rm += Point;
            Ixx += x*x+y*y*z*z; //todo calculus of the moment of inertia
          }
        }
      } 
    }
    
  V = V*dV;
  rm= rm*dV/V;
  Ixx = Ixx*dV;
  Ixx-= V*rm*rm;
  
  //! mass
  double pho=1; 
  m_m= pho*V ;
   
  //!moment of inertia
  m_Ixx = pho*Ixx;
  m_Iyy = pho*Ixx;
  m_Izz = pho*Ixx;

  //!center of mass
  m_r= rm;
  m_r1=rm;
  m_r2=rm;

  //! calculated of the radius of the bounding sphere with centered
  //! at the center of mass of the particle 
  double MAXL=0;
  Vec3 R0;
   
  for (it=m_ve.begin();it!=m_ve.end();++it)
  { 
    R0 = *it - rm;
    MAXL= max(MAXL,R0*R0);
  }
  m_dmax = sqrt(MAXL)+m_R; //todo check!  
}


void CParticle::updateVertices() {
    int i;
    Vec3 Zero=Vec3(0,0,0);
    //!each new face is obtained from the rotation and translation of the old Face
    for (i=0;i<m_Nf;i++) {
//        m_Face[i]=rotation_face(m_Faceold[i],m_Q,m_r);
          m_Face[i]=rotation_face(m_Faceold[i],m_Q,Zero);
          m_Face[i].translation(m_r);
    }
    //!each new edge is obtained from the rotation and translation of the old Face
    for (i=0;i<m_Ne;i++) {
//        m_Edge[i]=rotation_edge(m_Edgeold[i],m_Q,m_r);
          m_Edge[i]=rotation_edge(m_Edgeold[i],m_Q,Zero);
          m_Edge[i].translate(m_r);
    }
    //!each new vertex is obtained from the rotation and translation of the old Face
    for (i=0;i<m_Nv;i++) {
//        m_ve[i]=rotate(m_Q,m_veold[i])+m_r;
          m_ve[i]=rotate(m_Q,m_veold[i])+m_r;
    }
     
}


void CParticle::updateOrientation(double dt) {

/*
    step 1: calculate torque Tb at time t 
    step 2: calculate omega(t) from the stored value ome(t-dt/2) as ome(t) = w(t-dt/2)+I^{-1} Tb dt/2
    step 3: get dq/dt using the equation: dq/dt = Q(q)*w/2. Q is a matrix consisting of components of quaterions.
    step 4: from the stored w(t-dt/2) update w(t+dt/2) using w(t+dt/2) = w(t-dt/2)+I^{-1} Tb dt  
    step 5: compute q(t+dt/2) using q(t+dt/2) = q(t) + dq/dt dt/2
    step 6: evaluating  (dq/dt)(t+dt/2) using the equation in step 3
    step 7: calculate q(t+dt) using q(t+dt) = q(t) + dt (dq/dt)(t+dt/2)
    step 8: renormalize the quaternion q(t+dt).
*/

    double q0,q1,q2,q3,wx,wy,wz;

    //!get components of the quaternions \todo it is not necessary to make a copy of them
    q0=0.5*m_Q.getscalar();
    q1=0.5*m_Q.getvector().X();
    q2=0.5*m_Q.getvector().Y();
    q3=0.5*m_Q.getvector().Z();

    //!step 1: this is the Euler equation of motion Tb = F(I,omega)
    m_Td=Vec3((m_T.X()+(m_Iyy-m_Izz)*m_fome.Y()*m_fome.Z())/m_Ixx,(m_T.Y()+(m_Izz-m_Ixx)*m_fome.X()*m_fome.Z())/m_Iyy,(m_T.Z()+(m_Ixx-m_Iyy)*m_fome.X()*m_fome.Y())/m_Izz);

    //!step 2: calculate omega(t) from omega(t=dt/2)
    m_ome=m_fome+0.5*dt*m_Td;

    //!get compoments of omega
    wx=m_ome.X();
    wy=m_ome.Y();
    wz=m_ome.Z();

    //!step 3: calculate dq/dt at time t
    Quaternion dq(-(q1*wx+q2*wy+q3*wz),q0*wx-q3*wy+q2*wz,q3*wx+q0*wy-q1*wz,-q2*wx+q1*wy+q0*wz),qm;

    //!step 4: calculate ome(t+dt/2)
    m_fome=m_fome+m_Td*dt;

    //!step5: calculate q(t+dt/2)
    qm=m_Q+dq*(0.5*dt);

    //!get the components of q(t+dt/2)
    q0=0.5*qm.getscalar();
    q1=0.5*qm.getvector().X();
    q2=0.5*qm.getvector().Y();
    q3=0.5*qm.getvector().Z();

    //!get the components of ome(t+dt/2)
    wx=m_fome.X();
    wy=m_fome.Y();
    wz=m_fome.Z();

    //!step 6: calculate (dq/dt)(t+dt/2)
    dq=Quaternion(-(q1*wx+q2*wy+q3*wz),q0*wx-q3*wy+q2*wz,q3*wx+q0*wy-q1*wz,-q2*wx+q1*wy+q0*wz);

    //!step 7: calculate q(t+dt)
    m_Q=(qm+dq*0.5*dt);

    //!step 8: normalize q
    m_Q.normalize();

    //!calculate rotational kinetic energy
    m_Erot=0.5*(m_Ixx*wx*wx+m_Iyy*wy*wy+m_Izz*wz*wz);

}

void CParticle::start(double dt) {
    m_r1=m_r-m_v*dt;
}

void CParticle::updatePosition(double dt) {
    int i;
    Vec3 t;

    //! integration algorithm  
    //! r(t+dt) =   2*r(t) - r(t-dt) + a*dt^2
    //! v(t+dt) =  (r(t+dt) - r(t-dt))/(2*dt)

    //! calculate r(t+dt)
    m_r2=2*m_r-m_r1+m_f*(dt*dt/m_m);
    //!calculate v(t+dt)
    m_v=0.5*(m_r2-m_r1)/dt;
  
    //!calculate dr \todo not required
    t=m_r2-m_r;

    //!update r(t+dt) and r(t-dt) for the next time step
    m_r1=m_r;
    m_r=m_r2;
   
    //!calculate the linear kinetic energy
    m_Ekin=0.5*m_m*dot(m_v,m_v);
}

void CParticle::moveBy(Vec3 &t) {
    int i;
    m_r+=t;
    m_r1+=t;
    updateVertices();
}

//!\todo distancePointParticle only work still do not consider inner points.
double CParticle::distancePointParticle(Vec3 & Point) {
    CEdge3d Edge;
    //!initializing distance between point and skeleton
    Vec3 Dx = Point - m_ve[0];
    double distance = Dx.norm();
    //!for all Faces 
    vector<CFace3d>::iterator ite;
    for (ite=m_Face.begin();ite!=m_Face.end();++ite)
    { 
        //!calculate distance point to Face
        Edge = distance_point_Face3d(Point,*ite);
        //!update minimal distance
        distance = min(distance,Edge.L() );
    }
    return distance;
}

//!\todo point Inside Polyhedra test to be calculated 
bool CParticle::pointIsInsidePolyhedra(Vec3 & Point)
{
  return false;
}

bool CParticle::pointIsInsideParticle(Vec3 & Point)
{
  if (pointIsInsidePolyhedra(Point)==true)  
    return true;
  else if ( distancePointParticle(Point) < m_R  )
    return true;
  else 
    return false;
}
