
#ifndef __PARTICLE_H
#define __PARTICLE_H

#include "Face3d.h"



class CParticle {
    protected:
        //!Number of faces, number of Edges, Number of Vertices
        int m_Nf,m_Ne,m_Nv;
        //!STL-Vector of Faces 
        vector<CFace3d> m_Face,m_Faceold;
        //!STL-Vector of Edges
        vector<CEdge3d> m_Edge,m_Edgeold;
        //!STL-Vector of Vertices 
        vector<Vec3> m_ve,m_veold;
        //!mass, bounding radious, momnet of inertica, Kinetic energy, max diameter
        double m_m,m_R,m_Ixx,m_Iyy,m_Izz,m_Ekin,m_Erot,m_dmax;
        //!center of mass, velocity force, torque, torque_aux, angular velocity, aux-angular velocity, aux positions
        Vec3 m_r,m_v,m_f,m_T,m_Td,m_ome,m_fome,m_r1,m_r2;
        //!quaternion representing the orientation of the particle
        Quaternion m_Q;
        //!m_large = True if the particle is labelled as "large"
        bool m_large;
    public:
        //!default constructor
        CParticle (void) {};
        //!constructor of particle from filename
        CParticle(const string & filename_particle);
        //!constructor of particles entering 
        //!vertices as vector of Vec3
        //!edges as vector of pairs of Vec3
        //!faces as vector of Vec3-vectors
        CParticle( const vector<Vec3>              & VerticesList,
                   const vector< pair<Vec3,Vec3> > & EdgesList , 
                   const vector< vector<Vec3> >    & FacesList   ); 
        //!constructor of particle by entering:
        /*! \todo only tetra is corrected
	    double l0 length
            double R0 radius
            Vec3   r0 position
            Vec3   v0 velocity
            Vec3   0 center of rotation (unknown) 
            Vec3   Q0 quaternion
            string p  ParticleType
        */
        CParticle(double l0, double m0, double R0,Vec3 &r0, Vec3 &v0,  Vec3 &O,Quaternion &Q0,string p);
        //!constructor of particle wall:
        /*!
	    double a,b,c dimensions
            double R0 radius
            Vec3   r0 position
            Vec3   Q0 quaternion
            Vec3   v0 velocity
        */
        CParticle(double a,double b,double R0,Vec3 &r0,Quaternion &Q0,const Vec3 & v0=Vec3(0,0,0));
        //!constructor of particle hopper:
        /*! \todo to correct
	    double a,b,c dimensions
            double R0 radius
            Vec3   r0 position
            Vec3   Q0 quaternion
        */
        CParticle(double a,double b,double c,double R0,Vec3 &r0,Quaternion &Q0);
        //!constructor of rough particle:
        /*! \todo to correct
	    double a,b dimensions
            int N number of lumps
            double R0 Radius
            double l0 heigh of lumps
            Vec3   r0 position
            Vec3   Q0 quaternion
        */
        CParticle(double a,double b,int N,double R0,double l0,Vec3 &r0,Vec3 &v0,Quaternion &Q0);
        //!calculation of mass properties \todo
        void calcMassProperties(const int N);
        //! update vertices
        void updateVertices();
        //void rotation_quaternion(Quaternion &);
        //! update of the quaternion at time step dt
        void updateOrientation(double dt);
        //!start dynamics of the particle 
        void start(double);
        //! update of the center of mass at time step dt
        void updatePosition(double);
        //! translation of the particle by a vector displacement
        void moveBy(Vec3 &);
        //! start force of the particle
        void startforce(const Vec3 & F=Vec3(0,0,0)) {m_f=F; m_T=Vec3(0,0,0);};
        //! set torque of the particle    
        void setT(const Vec3 & F=Vec3(0,0,0)) {m_T=F;};
        //! set mass of  the particle
        void setm(double m) {m_m=m;};
        //! set velocity of the particle
        void setv(const Vec3 & F=Vec3(0,0,0)) {m_v=F;};
        //! get maximal x
        double getXmax(void) {
            double x=getVer(0).X();
            for (int i=1;i<m_Nv;i++) {
                if (getVer(i).X()>x) x=getVer(i).X();
            }
            return x+m_R;
        }
        //! get minimal x
        double getXmin(void) {
            double x=getVer(0).X();
            for (int i=1;i<m_Nv;i++) {
                if (getVer(i).X()<x) x=getVer(i).X();
            }
            return x-m_R;
        }
        //! return linearkinetic energy
        double getEkin(void) {return m_Ekin;};
        //! return rotational kinetic energy
        double getErot(void) {return m_Erot;};
        //! return mass
        double getm(void) {return m_m;};
        //! return xx-component of moment of inertia
        double getIxx(void) {return m_Ixx;};
        //! return yy-component of moment of inertia
        double getIyy(void) {return m_Iyy;};
        //! return zz-component of moment of inertia
        double getIzz(void) {return m_Izz;};
        //! return diameter 
        double getdmax(void) {return m_dmax;};
        //! return power done by the particle
        double getPower(void) {return dot(m_f,m_v)+dot(m_T,m_ome);};
        //! return vertex with index i
        Vec3 getVer(int i) {return m_ve[i];};
        //! return position
        Vec3 getr(void) {return m_r;};
        //! return angular momentum
        Vec3 getome(void) {return m_ome;}; 
        //! return aux-angular momentum (unknown)
        Vec3 getfome(void) {return m_fome;};
        //! return velocity
        Vec3 getv(void) {return m_v;};
        //! return torque
        Vec3 getT(void) {return m_T;};
        //! return Torque
        Vec3 getTd(void) {return m_Td;};
        //! return force
        Vec3 getF(void) {return m_f;};
        //! return momentum
        Vec3 getp(void) {return m_m*m_v;};
        //! return angular momentum
        Vec3 getL(void) {return Vec3(m_Ixx*m_ome.X(),m_Iyy*m_ome.Y(),m_Izz*m_ome.Z());};
        //! returns aux-angular momentum (unkwnown)
        Vec3 getLs(void) {Vec3 v=getL(); v=m_m*cross(m_r,m_v)+rotate(m_Q,v); return v;};
        //! returns vertex (already defined)
        Vec3 getvertex(int i) {return m_ve[i];};
        //! returns quaternions
        Quaternion getQ(void) {return m_Q;};
        friend class CGraph;
        friend class CInteracton;
        //!distance point to Particle \todo constaintization
        double distancePointParticle(Vec3 & Point) ;
        //!point is inside the  skeleton of the particle \todo constainization returns always false
        bool pointIsInsidePolyhedra(Vec3 & Point);
        //!point is inside particle \todo constainization
        bool pointIsInsideParticle(Vec3 & Point);
};


#endif
